package org.foo.hello.extender;

import java.util.Dictionary;

import org.foo.hello.Greeting;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleEvent;
import org.osgi.framework.BundleListener;

public class GreetingListener implements BundleListener {

	private GreetingSet m_gs = null;

	public GreetingListener(GreetingSet gs) {
		m_gs = gs;
	}

	private boolean isExtension(Bundle bundle) {
		Dictionary dict = bundle.getHeaders();

		// Try to get the name of the extension.
		String name = (String) dict.get(Greeting.NAME_PROPERTY);
		// Return immediately if the bundle is not an extension.
		if (name == null) {
			System.out.println("Bundle " + bundle.getSymbolicName()
					+ " not a Greeting Extension");
			return false;
		}
		System.out.println("Bundle " + bundle.getSymbolicName()
				+ " is a Greeting Extension");
		return true;
	}

	private void addExtension(Bundle bundle) {
		System.out.println("Greeting extension will be added: "
				+ bundle.getSymbolicName());
		Dictionary dict = bundle.getHeaders();
		String name = (String) dict.get(Greeting.NAME_PROPERTY);
		// Get the class of the extension.
		String className = (String) dict.get(Greeting.CLASS_PROPERTY);
		Class clazz = null;
		try {
			clazz = bundle.loadClass(className);
			m_gs.addGreeting(name, (Greeting) clazz.newInstance());
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void removeExtension(Bundle bundle) {
		System.out.println("Greeting extension will be removed: "
				+ bundle.getSymbolicName());
		Dictionary dict = bundle.getHeaders();
		String name = (String) dict.get(Greeting.NAME_PROPERTY);
		m_gs.removeGreeting(name);
	}

	public void bundleChanged(BundleEvent event) {
		System.out.println("Listener: Event type=" + event.getType()
				+ " bundle" + event.getBundle() + " source="
				+ event.getSource());
		Bundle bundle = event.getBundle();

		if (event.getType() == BundleEvent.STARTED) {
			if (isExtension(bundle))
				addExtension(bundle);
		}

		if (event.getType() == BundleEvent.STOPPED) {
			if (isExtension(bundle))
				removeExtension(bundle);
		}
	}
}